/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#ifndef DJ_AUDIO_BUFFER_H
#define DJ_AUDIO_BUFFER_H

#include "DJTypes.h"

///////////////////////////////////////////////////////
// This class stands in close relationship to        //
// AudioFile, thats why it's implementation .txx     //
// is included there. This prevents a cyclic include //
///////////////////////////////////////////////////////

namespace DJ
{

/**
 * \class AudioBuffer
 * \brief Ringbufffer for type T
 *
 * This class provides a ringbuffer for the specified type T.
 * It is mainly used to store audio read from an audiofile.
 */
template<typename T>
class AudioFile::AudioBuffer
{
    public:
        /**
         * \brief Initialize ringbuffer from file
         * \param file -
         * \param mode -
         * \param start -
         * \param size - the maximal size of the ringbuffer
         * \param window - optional, sets the window which moves in
                    the buffer   |  <-[  ]->     |
         */
        AudioBuffer(
            const AudioFile* file,
            const AudioBufferMode mode,
            const CountT start,
            const CountT size,
            const CountT window=0);

        /**
         * \brief Initialize ringbuffer from buffer
         * \param buf -
         * \param start -
         * \param size - the maximal size of the ringbuffer
         * \param window - optional, sets the window which moves in
                    the buffer   |  <-[  ]->     |
         */
        AudioBuffer(
            const AudioBuffer<T>* buf,
            const CountT start,
            const CountT size,
            const CountT window=0);

        /**
         * \brief Destructor
         * Destoys read data if the buffer is defined to be the parent
         */
        virtual ~AudioBuffer();

        /**
         * \brief Get data
         * Get data at given position.
         * Accessing and reloading is handled transparently;
         * \param at - position in the buffer to retrieve
         */
        virtual const T& Get(const CountT at) const;

        virtual CountT Size() const;
        virtual unsigned short Channels() const;
        virtual SampleRateT SampleRate() const;

    private:
        struct D; D* data;
};

}

#endif
