/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#ifndef DJ_AUDIOFILE_H
#define DJ_AUDIOFILE_H

#include "DJTypes.h"
#include "DJMetaData.h"
#include <string>

namespace DJ
{

enum AudioBufferMode
{
    FRAMES,
    ITEMS
};

enum AudioFileTag
{
    TITLE       = 0x01,
    COPYRIGHT   = 0x02,
    SOFTWARE    = 0x03,
    ARTIST      = 0x04,
    COMMENT     = 0x05,
    DATE        = 0x06,
    ALBUM       = 0x07
};

class AudioFile
{
    public:

        template<typename T>
        class AudioBuffer;

        AudioFile(std::string path);
        virtual ~AudioFile();

        virtual void ReadFrames(const CountT start, const CountT items, short* target) const;
        virtual void ReadFrames(const CountT start, const CountT items, int* target) const;
        virtual void ReadFrames(const CountT start, const CountT items, float* target) const;
        virtual void ReadFrames(const CountT start, const CountT items, double* target) const;

        virtual void Read(const CountT start, const CountT items, short* target) const;
        virtual void Read(const CountT start, const CountT items, int* target) const;
        virtual void Read(const CountT start, const CountT items, float* target) const;
        virtual void Read(const CountT start, const CountT items, double* target) const;


        template<typename T>
        AudioBuffer<T>* CreateBuffer(
            const CountT start,
            const CountT items,
            const CountT window = 0,
            const AudioBufferMode mode = FRAMES
        );

        virtual std::string GetPath() const;

        virtual const char* GetFileTag(const AudioFileTag& tag) const;

        virtual CountT GetFrameCount() const;
        virtual SampleRateT GetSampleRate() const;
        virtual unsigned short GetChannelCount() const;
        virtual FormatT GetFormat() const;
        virtual CountT GetSectionCount() const;
        virtual bool IsSeekable() const;

    private:
        struct D; D* data;
};

template<typename T>
AudioFile::AudioBuffer<T>*
AudioFile::CreateBuffer(
    const CountT start,
    const CountT size,
    const CountT window,
    const AudioBufferMode mode
)
{
    CountT w = window ? window : size;
    AudioBuffer<T>* buf = new AudioBuffer<T>(this, mode, start, size, w);
    return buf;
}

}

///////////////////////////////////////////////////////
// This class stands in close relationship to        //
// AudioFile, thats why it's implementation .txx     //
// is included here. This prevents a cyclic include  //
///////////////////////////////////////////////////////

#include "DJAudioBuffer.h"

#ifndef DJ_MANUAL_INSTANTIATION
    #include "DJAudioBuffer.txx"
#endif

#endif
