/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include "DJExtractMetaData.h"

#include "DJLogger.h"
#include "DJMetaDataChooser.h"

#include <QMessageBox>

#include <assert.h>
#include <vector>

#include <musicbrainz3/webservice.h>
#include <musicbrainz3/query.h>
#include <musicbrainz3/model.h>

#include <iostream>

#include <QUrl>

namespace DJ
{


struct ExtractMetaData::D
{
    Source<AudioFile>* input;
    std::vector< Sink<MetaData>* > connections;

    AudioFile*  file;
    MetaData* out;

    Logger* log;
};


ExtractMetaData::ExtractMetaData()
{
    data = new D;
    data->log = Logger::GetInstancePtr();
    data->file = 0;
    data->input = 0;
    data->out = new MetaData();;

    *data->log << "ExtractMetaData:\tInitializing" << Log::NLn;
}



ExtractMetaData::~ExtractMetaData()
{
    *data->log << "ExtractMetaData:\tDestructing" << Log::NLn;
    data->log->finish();
    delete data;
}


void
ExtractMetaData::Refine()
{
#if 0
    assert(data->file);
    AudioFile::AudioBuffer<float>* buf = data->file->CreateBuffer<float>(
            0,
            data->file->GetFrameCount(),
            50000,
            FRAMES
        );

    const unsigned short CHANNELS = buf->Channels();
    const SampleRateT SAMPLERATE = buf->SampleRate();

    const CountT MIN_LENGTH = 40*SAMPLERATE; // minimal tracklength is 40s
    const CountT MIN_PAUSE = 2*SAMPLERATE; // minimal pause length is 2s
    const CountT MAX_PAUSE = 6*SAMPLERATE; // maximal pause length is 5s

    static const double SILENCE = 0.08; //Silence Threshold;

    std::vector<TrackInfo> test_vec;

    CountT silence_count = 0;
    CountT track_count = 0;
    TrackInfo track;
    track.side = 1;
    track.title = "Automatic";
    bool silence = true;
    static const unsigned short  WINDOW = SAMPLERATE/100;
    double block_max = -0.;
    double block_min = 0.;
    CountT count = 0;
    int eval=0;
    int max_track=50;
    int max_silence=-50;
    ms mark = 0;
    for(CountT pos = 0; pos < data->file->GetFrameCount(); pos++)
    {
        //peaks
        if(count >= WINDOW)
            count = 0;
        else
        {
            double val = buf->Get(pos);
            block_max = val > block_max ? val : block_max;
            block_min = val < block_min ? val : block_min;
            count++;
            continue;
        }

        double sek = (double)pos/(double)SAMPLERATE;
//        std::cout << "NEW Peak: " << block_max << "\t\t" << block_min << std::endl;

        //check threshold
        bool silent = (block_max < SILENCE) && (block_min > -SILENCE);

        std::cout <<block_max << " " << block_min << "    " <<silent << std::endl;

        if(silent)
        {
            if(eval == max_track)
                mark =(double)pos/(double)SAMPLERATE * 1000.;
            eval = eval == max_silence ? eval : eval-1;
        }
        else
        {
            if(eval == max_silence)
                mark = (double)pos/(double)SAMPLERATE * 1000.;
            eval = eval == max_track ? eval : eval+1;
        }

        if(eval == max_track)
            std::cout << "TRACK "<< mark <<std::endl;
        if(eval == max_silence)
            std::cout << "SILENCE "<< mark << std::endl;

        block_max = 0;
        block_min = 0;
    }

    for(unsigned i=0; i<test_vec.size(); i++)
    {
        TrackInfo t = test_vec[i];
        *data->log << "REFINING - TRACK: " << i << " Start: " << t.start << " Stop: " << t.stop << Log::NLn;
    }

    delete buf;
#endif
}

void
ExtractMetaData::Query(
    const MetaDataEngine& engine,
    const SearchCriteria* criteria)
{
    switch(engine)
    {
    case MusicBrainz:
        QueryMusicBrainz(criteria);
        break;
    default:
        break;
    }
}

void
ExtractMetaData::QueryMusicBrainz(const SearchCriteria* criteria)
{
    assert(data->out);
    if(!criteria)
        return;

    int index;
    MetaDataChooser* chooser = new MetaDataChooser(&index);

    if(criteria->Album == "" || criteria->Artist == "")
        return;

    MusicBrainz::Query query;
    MusicBrainz::ReleaseResultList releases;

    try {
        releases = query.getReleases(
            &MusicBrainz::ReleaseFilter().artistName(criteria->Artist).title(criteria->Album)
        );
    } catch(...) {
        QMessageBox::warning ( 0, "Error", "Could not retrieve metadata. Is a internet connection established?");
        return;
    }
    std::vector<MetaOption> options;

    for (MusicBrainz::ReleaseResultList::iterator i = releases.begin(); i != releases.end(); i++)
    {
        QString option ="";
        MetaOption metaoption;

        MusicBrainz::ReleaseResult *result = *i;
        MusicBrainz::Release *release;
        try {
            release = query.getReleaseById(result->getRelease()->getId(), &MusicBrainz::ReleaseIncludes().tracks().artist());
        }
        catch (MusicBrainz::WebServiceError &e) {
            *data->log << "Error: " << e.what() << Log::NLn;
            continue;
        }

        option += "Id: " + QString::fromUtf8(release->getId().data(), release->getId().length())  + "\n";

        metaoption.meta.Album = release->getTitle();
        option += "Title: " + QString::fromUtf8(release->getTitle().data(), release->getTitle().length()) + "\n";

        metaoption.meta.Artist = release->getArtist()->getName();
        option += "Artist: " + QString::fromUtf8(release->getArtist()->getName().data(), release->getArtist()->getName().length()) + "\n";
        option += "Tracks:\n    ";
        int trackno = 1;
        ms start = 0;
        for (MusicBrainz::TrackList::iterator j = release->getTracks().begin(); j != release->getTracks().end(); j++) {
            MusicBrainz::Track *track = *j;

            TrackInfo trackinf;
            trackinf.side = 1;
            trackinf.start = start;
            trackinf.stop = start+ track->getDuration();
            trackinf.title = track->getTitle();
            metaoption.tracks.push_back(trackinf);
            start = trackinf.stop+2;

            option += QString::number(trackno,10) + ". " + QString::fromStdString(track->getTitle()) + "\n";
            option += "    ";
            trackno++;
        }

        options.push_back(metaoption);
        chooser->AddOption(option);

        delete result;
    }

    chooser->exec();

    if(index > -1)
    {
        data->out->SetAlbum(options[index].meta.Album);
        data->out->SetArtist(options[index].meta.Artist);

        data->out->AccessTrackList()->Clear();
        ms length = ((long double)data->file->GetFrameCount()/(long double)data->file->GetSampleRate()) *1000.;
        ms delta = 0;
        for(unsigned short i=0; i<options[index].tracks.size(); i++)
        {
            if(options[index].tracks[i].stop >= length)
            {
                options[index].tracks[i].side = 2;
                delta = delta ? delta : options[index].tracks[i].start;
            }

            options[index].tracks[i].start -= delta;
            options[index].tracks[i].stop -= delta;

            data->out->AccessTrackList()->AddTrack(options[index].tracks[i]);
        }

        data->out->SetFocus(0);
    }
}


MetaData*
ExtractMetaData::GetOutput()
{
    assert(data->input);
    AudioFile* f = data->input->GetOutput();
    assert(f);
    if(f != data->file || !data->out)
    {
        *data->log << "ExtractMetaData: Creating Metadata"<< Log::NLn;
        data->file = f;
        if(data->out)
            delete data->out;

        data->out = new MetaData();
    }

    return data->out;
}

void
ExtractMetaData::Update()
{
    if(data->out)
        delete data->out;
    data->out = 0;

    for(unsigned short i = 0; i<data->connections.size(); i++)
        data->connections[i]->Update();
}

void
ExtractMetaData::SetInput(
    Source<AudioFile>* input
)
{
    assert(input);
    data->input = input;
    data->file = input->GetOutput();
}

void
ExtractMetaData::connect(
    Sink<MetaData>* out
)
{
    out->SetInput( this );
    data->connections.push_back(out);
}

}

