/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

 
#ifndef DJ_EXTRACTMETADATA_H
#define DJ_EXTRACTMETADATA_H

#include "DJPipeline.h"
#include "DJAudioFile.h"
#include "DJTypes.h"
#include <string>

namespace DJ
{

enum MetaDataEngine
{
    MusicBrainz = 0x00
};

struct SearchCriteria
{
    std::string Album;
    std::string Artist;
    std::string Year;
    std::string Genre;
    unsigned short Tracks;
    ms Length;
};

class ExtractMetaData :
    public Filter<AudioFile, MetaData >
{
    public:
        ExtractMetaData();
        virtual ~ExtractMetaData();

        /**
         * \see Source::GetOutput()
         */
        virtual MetaData* GetOutput();

        /**
         *  \brief Try to refine tracks
         */
        void Refine();

        /**
         * \brief Query Searchengine
         * Query the given engine using the given search criteria in
         * order to retrieve additional metadata like tracklist etc.
         * \param engine the searchengine to query
         * \param criteria the search criteria
         */
        void Query(const MetaDataEngine& engine, const SearchCriteria* criteria = 0);

        /**
         * \see Sink::SetInput()
         */
        virtual void SetInput(Source<AudioFile>*);

        /**
         * \see Source::Update()
         */
        virtual void Update();

    protected:
        /**
         * \brief Qery Musicbrainz
         * \see ExtractMetaData::Query
         */
        virtual void QueryMusicBrainz(const SearchCriteria*);

        /**
         * \see Source::connect
         */
        void connect( Sink<MetaData>* out );

        struct MetaOption
        {
            SearchCriteria meta;
            std::vector<TrackInfo> tracks;
        };

    private:
        struct D; D* data;

};

}

#endif
