/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include "DJMetaData.h"

#include <iostream>

namespace DJ
{

struct MetaData::D
{
    std::string album;
    std::string artist;
    std::string genre;
    std::string year;
    std::string comment;
    short focus;
    TrackList tracks;
};

MetaData::MetaData(QObject * parent)
: QObject(parent)
{
    data = new D;
    data->focus = -1;
    connect(&data->tracks, SIGNAL(TrackRemoved(unsigned short)),
            this, SLOT(TrackRemoved(unsigned short)));
}

MetaData::~MetaData()
{
    delete data;
}

const std::string&
MetaData::GetAlbum() const
{
    return data->album;
}

const std::string&
MetaData::GetArtist() const
{
    return data->artist;
}

const std::string&
MetaData::GetGenre() const
{
    return data->genre;
}

const std::string&
MetaData::GetYear() const
{
    return data->year;
}

const std::string&
MetaData::GetComment() const
{
    return data->comment;
}

const short&
MetaData::GetFocus() const
{
    return data->focus;
}

const TrackList*
MetaData::GetTrackList() const
{
    return &data->tracks;
}

TrackList*
MetaData::AccessTrackList()
{
    return &data->tracks;
}

void
MetaData::SetAlbum(std::string val)
{
    data->album = val;
    emit MetaDataChanged(Album, val);
}
void
MetaData::SetAlbum(QString val)
{
//     std::string str( val.toUtf8().data() );
    std::string str = MetaData::QStringToStdString(val);
    SetAlbum(str);
}

void
MetaData::SetArtist(std::string val)
{
    data->artist = val;
    emit MetaDataChanged(Artist, val);
}
void
MetaData::SetArtist(QString val)
{
//     std::string str( val.toUtf8().data() );
    std::string str = MetaData::QStringToStdString(val);
    SetArtist(str);
}

void
MetaData::SetGenre(std::string val)
{
    data->genre = val;
    emit MetaDataChanged(Genre, val);
}
void
MetaData::SetGenre(QString val)
{
//     std::string str( val.toUtf8().data() );
    std::string str = MetaData::QStringToStdString(val);
    SetGenre(str);
}

void
MetaData::SetYear(std::string val)
{
    data->year = val;
    emit MetaDataChanged(Year, val);
}
void
MetaData::SetYear(QString val)
{
//     std::string str( val.toUtf8().data() );
    std::string str = MetaData::QStringToStdString(val);
    SetYear(str);
}

void
MetaData::SetComment(std::string val)
{
    data->comment = val;
    emit MetaDataChanged(Comment, val);
}
void
MetaData::SetComment(QString val)
{
//     std::string str( val.toUtf8().data() );
    std::string str = MetaData::QStringToStdString(val);
    SetComment(str);
}

void
MetaData::SetFocus(short focus)
{
    if(focus != data->focus)
    {
        data->focus = focus;
        emit FocusChanged(focus);
    }
}

void
MetaData::TrackRemoved(unsigned short track)
{
    SetFocus(track-1);
}

}
