/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#ifndef DJ_METADATA_H
#define DJ_METADATA_H

#include <QObject>
#include <string>
#include <iostream>
#include "DJTrackList.h"

namespace DJ
{

enum MetaDataTag
{
    Album,
    Artist,
    Genre,
    Year,
    Comment
};

class MetaData :
    public QObject
{
    Q_OBJECT;

    public:
        static std::string QStringToStdString(QString str)
        {
            std::string s(str.toUtf8().data());
            return s;
        }

        static QString StdStringToQString(std::string str)
        {

            return QString::fromUtf8(str.data());
        }

        MetaData(QObject * parent = 0 );
        ~MetaData();

        const std::string& GetAlbum() const;
        const std::string& GetArtist() const;
        const std::string& GetGenre() const;
        const std::string& GetYear() const;
        const std::string& GetComment() const;
        const short& GetFocus() const;
        const TrackList* GetTrackList() const;

        TrackList* AccessTrackList();

    public slots:
        void SetAlbum(std::string);
        void SetArtist(std::string);
        void SetGenre(std::string);
        void SetYear(std::string);
        void SetComment(std::string);

        void SetAlbum(QString);
        void SetArtist(QString);
        void SetGenre(QString);
        void SetYear(QString);
        void SetComment(QString);

        void SetFocus(short);

    protected slots:
        void TrackRemoved(unsigned short);

    signals:
        void MetaDataChanged(MetaDataTag, std::string);
        void FocusChanged(short);

    private:
        struct D; D* data;
};

}

#endif
