/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include "DJTrackList.h"

#include <vector>
#include <iostream>

using std::vector;

namespace DJ
{
struct TrackList::D
{
    vector< TrackInfo > tracks;
};

TrackList::TrackList(QObject * parent)
: QObject(parent)
{
    data = new D;
}

TrackList::~TrackList()
{
    delete data;
}

unsigned
TrackList::AddTrack(
    const TrackInfo info
)
{
    data->tracks.push_back(info);
    unsigned short pos = data->tracks.size()-1;
    emit TrackChanged(pos);
    return pos;
}

unsigned
TrackList::AddTrack(
    unsigned short side,
    std::string title,
    ms start,
    ms stop
)
{
    TrackInfo t;
    t.side = side;
    t.title = title;
    t.start = start;
    t.stop = stop;
    return AddTrack(t);
}

void
TrackList::EditTrack(
    const unsigned short track,
    TrackInfo info
)
{
    data->tracks[track] = info;
    emit TrackChanged(track);
}

void
TrackList::RemTrack(
    const unsigned short track
)
{
    if(track >= data->tracks.size())
        return;

    data->tracks.erase(data->tracks.begin()+track);

    emit TrackRemoved(track);
}

void
TrackList::SwapTracks(
    const unsigned short from,
    const unsigned short to
)
{
    TrackInfo tmp = data->tracks[from];

    data->tracks[from] = data->tracks[to];
    data->tracks[to] = tmp;

    emit TrackChanged(from);
    emit TrackChanged(to);
}

unsigned
TrackList::Size() const
{
    return data->tracks.size();
}

const TrackInfo*
TrackList::Begin() const
{
    return &data->tracks[0];
}

const TrackInfo*
TrackList::End() const
{
    return &data->tracks[data->tracks.size()-1];
}

const TrackInfo*
TrackList::At(
    const unsigned short track
) const
{
    return &data->tracks.at(track);
}

void
TrackList::Clear()
{
    for(unsigned short i=0; i< data->tracks.size(); i++)
        RemTrack(i);

    data->tracks.clear();
}

}
