/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include "DJManageFiles.h"
#include "conf.h"

#include <QPlainTextEdit>

#include <QFileDialog>

#include <iostream>
#include <sstream>
#include <stdlib.h>

namespace DJ
{

ManageFiles::ManageFiles()
{
    setupUi(this);
    connect(pbFileA, SIGNAL(clicked()), this, SLOT(FileA()));
    connect(pbFileB, SIGNAL(clicked()), this, SLOT(FileB()));
    connect(pbMp3ToWav, SIGNAL(clicked()), this, SLOT(ConvertFiles()));
}

std::string ManageFiles::GetFileA() const
{
    return std::string(leFileA->text().toUtf8().data());
}

std::string ManageFiles::GetFileB() const
{
    return std::string(leFileB->text().toUtf8().data());
}

void ManageFiles::FileA()
{
    QString file = QFileDialog::getOpenFileName(this, "Choose a file", HOME_DIR, "Audiofile File (*.wav *.mp3)");
    if(!file.isNull())
    {
        leFileA->setText(file);
    }
}

void ManageFiles::FileB()
{
    QString file = QFileDialog::getOpenFileName(this, "Choose a file", HOME_DIR, "Audiofile (*.wav *.mp3)");
    if(!file.isNull())
    {
        leFileB->setText(file);
    }
}

void ManageFiles::ConvertFiles()
{
    blockSignals(true);
    QString file = QFileDialog::getOpenFileName(this, "Choose file to convert", HOME_DIR, "Mp3 File (*.mp3)");
    if(!file.isNull())
    {
        QString nfile = QFileDialog::getSaveFileName(this, "Where should the converted file be stored?", HOME_DIR, "Wav File (*.wav)");
        if(!nfile.isNull())
        {
            lblOut->setText("Processing... You will be asked were to store the wav for each file.");
            std::ostringstream cmd (std::ostringstream::out);
            cmd << DJ_LAME << " --decode \"" << file.toUtf8().data() << "\" \"" << nfile.toUtf8().data() << "\"" << std::endl;
            system(cmd.str().data());
        }
        else
        {
            blockSignals(false);
            return;
        }
    }
    else
    {
        blockSignals(false);
        return;
    }
    
    blockSignals(false);
    
    lblOut->setText("Done");
}

}
