/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#ifndef DJ_TRACKSTASK_H
#define DJ_TRACKSTASK_H

#include <ui_tracks_task.h>
#include <QWidget>
#include <string>
#include "DJTypes.h"
#include "DJMetaData.h"
#include "DJReadAudioFile.h"
#include "DJExtractMetaData.h"

namespace DJ
{

class TracksTask :
    public QWidget,
    private Ui::TracksTask
{
    Q_OBJECT;

    public:
        TracksTask();
        virtual ~TracksTask();
        virtual void Manage();

    signals:
        void PublishMeta(ExtractMetaData*);
        void PublishReaderA(ReadAudioFile*);
        void PublishReaderB(ReadAudioFile*);

    protected:
        virtual void SetupActions();

    protected slots:
        virtual void AddTrack();
        virtual void RemTrack();
        virtual void MvTrackUp();
        virtual void MvTrackDown();
        virtual void PlayTrack(bool);
        virtual void Play(ms start, ms stop, unsigned short side);
        virtual void Stop();
        virtual void SelectTrack(int,int,int,int);
        virtual void TrackEdit(int,int);
        virtual void MetaDataChanged(MetaDataTag tag, std::string val);
        virtual void TrackRemoved(unsigned short track);
        virtual void TrackListChanged(unsigned short track);
        virtual void RetrieveMetaData();
        virtual void Refine();
        virtual void FocusChanged(short);
        virtual void ShowOffset();
        virtual void ShowGap();

        virtual void SideASelected();
        virtual void SideBSelected();

    private:
        struct D; D* data;
};

}

#endif
