/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#ifndef DJ_WAVEFORMMARKER_H
#define DJ_WAVEFORMMARKER_H

#include <QObject>
#include "DJPipeline.h"
#include "DJWaveFormScene.h"

#include "DJTypes.h"

namespace DJ
{

class MetaData;

class WaveFormMarker :
    public QObject,
    public Sink<MetaData>
{
    Q_OBJECT;
    public:
        WaveFormMarker(
            WaveFormSceneCommon* scene,
            const unsigned short side,
            QObject* parent=0
        );
        virtual ~WaveFormMarker();

        /**
         * \see Sink::SetInput
         */
        virtual void SetInput(Source< MetaData >*);

        /**
         *  \see Sink::Update
         */
        virtual void Update();


    public slots:
        virtual void SetPos(ms);

    protected slots:
        virtual void SetFocus(short);
        virtual void UpdateTrack(unsigned short);
        virtual void RemoveTrack(unsigned short);
        virtual void Update(QRectF);

    protected:
        virtual void PropagateNewPos(ms);

    signals:
        void NewPos(ms);

    private:
        class GraphicTrack;
        struct D; D* data;
};

}

#endif
