/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#ifndef DJ_WAVEFORMSCENE_H
#define DJ_WAVEFORMSCENE_H

#include <QGraphicsScene>

#include "DJTypes.h"
#include "DJAudioFile.h"

class QGraphicsSceneWheelEvent;
class QGraphicsSceneDragDropEvent;

namespace DJ
{

class WaveFormSceneCommon :
    public QGraphicsScene
{
    Q_OBJECT;

    public:
        WaveFormSceneCommon(QObject* parent=0) : QGraphicsScene(parent) {}
        virtual ~WaveFormSceneCommon() {}

        virtual qreal PixelForPos(const ms& pos) const = 0;
        virtual ms PosForPixel(const qreal& pixel) const = 0;

    signals:
        void Selected();

    protected:
        //overloaded interactions
        virtual void wheelEvent ( QGraphicsSceneWheelEvent * wheelEvent ) = 0;

        virtual void focusInEvent ( QFocusEvent *)
        {
            emit Selected();
        }

};

template <typename T>
class WaveFormScene :
    public WaveFormSceneCommon
{
    public:
        WaveFormScene(
            const qreal initWidth,
            const qreal initHeight,
            QObject* parent=0
        );
        virtual ~WaveFormScene();

        virtual void SetAmplitudes(
            const AudioFile::AudioBuffer<T>* amplitudes
        );
        virtual void ReDraw();

        virtual qreal PixelForPos(const ms& pos) const;
        virtual ms PosForPixel(const qreal& pixel) const;

    protected:
        //overloaded interactions
        virtual void wheelEvent ( QGraphicsSceneWheelEvent * wheelEvent );


    private:
        struct D; D* data;
};

}

#ifndef DJ_MANUAL_INSTANTIATION
    #include "DJWaveFormScene.txx"
#endif

#endif
