/*
Copyright (c) 2009, Christian Junker
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the project nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY CHRISTIAN JUNKER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL CHRISTIAN JUNKER BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#ifndef DJ_LOGGER_H
#define DJ_LOGGER_H

#include <string>
#include <vector>

#include <QReadWriteLock>
#include <QThread>

#include "DJLog.h"

namespace DJ
{

/**
 * @class Logger
 * @brief Loggingfacility
 */
class Logger :
    public QThread
{
    public:
        static Logger& GetInstance();
        static Logger* GetInstancePtr();

        static unsigned AddOutput(Log * out);

        static void RemOutput(const unsigned& id, bool del=false);
        static void RemOutput(Log * out, bool del=false);

        Logger& operator<< (bool& val);
        Logger& operator<< (short& val);
        Logger& operator<< (unsigned short& val);
        Logger& operator<< (int& val);
        Logger& operator<< (unsigned int& val);
        Logger& operator<< (long& val);
        Logger& operator<< (unsigned long& val);
        Logger& operator<< (float& val);
        Logger& operator<< (double& val);
        Logger& operator<< (long double& val);
        Logger& operator<< (long long& val);

        Logger& operator<< (const NLnType&) { *this << m_NewLine; return *this; }

        Logger& operator<<(std::string msg);

        void finish();

    protected:
        template<typename T>
        std::string Prepare(const T& val) const;

        void run();

    private:
        Logger();
        ~Logger();
        Logger(const Logger&); //purposely not implemented
        void operator=(const Logger&); //purposely not implemented

        static Logger *     m_Self;
        static unsigned     m_Ref;

        static std::vector<Log *>   *m_Logs;
        static QReadWriteLock        m_LogMutex;

        static const std::string    m_NewLine;

        struct D; D* data;

};

}

#endif
